'use strict';

exports.__esModule = true;

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _gittar = require('gittar');

var _gittar2 = _interopRequireDefault(_gittar);

var _fs = require('fs.promised');

var _fs2 = _interopRequireDefault(_fs);

var _chalk = require('chalk');

var _path = require('path');

var _inquirer = require('inquirer');

var _asyncCommand = require('../utils/async-command');

var _asyncCommand2 = _interopRequireDefault(_asyncCommand);

var _utils = require('../utils');

var _setup = require('../utils/setup');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const TEMPLATE = 'skpm/skpm';
const RGX = /\.(woff2?|ttf|eot|jpe?g|ico|png|gif|mp4|mov|ogg|webm)(\?.*)?$/i;
const isMedia = str => RGX.test(str);

exports.default = (0, _asyncCommand2.default)({
  command: 'create <dest>',

  desc: 'Create a new plugin.',

  builder: {
    cwd: {
      description: 'A directory to use instead of $PWD.',
      type: 'string',
      default: '.'
    },
    name: {
      description: "The application's name",
      type: 'string'
    },
    template: {
      description: 'The repository hosting the template to start from',
      type: 'string',
      default: 'skpm/skpm'
    },
    force: {
      description: 'Force option to create the directory for the new app',
      type: 'boolean',
      default: false
    },
    git: {
      description: 'Initialize version control using git',
      type: 'boolean',
      default: true
    },
    install: {
      description: 'Install dependencies',
      type: 'boolean',
      default: true
    }
  },

  handler(argv) {
    return _asyncToGenerator(function* () {
      // Prompt if incomplete data
      if (!argv.dest) {
        (0, _utils.warn)('Insufficient command arguments! Prompting...');
        (0, _utils.info)('Alternatively, run `skpm create --help` for usage info.');

        const questions = (0, _setup.isMissing)(argv);
        const response = yield (0, _inquirer.prompt)(questions);
        Object.assign(argv, response);
      }

      if (!argv.name) {
        argv.name = argv.dest; // eslint-disable-line
      }

      const cwd = (0, _path.resolve)(argv.cwd);
      const target = argv.dest && (0, _path.resolve)(cwd, argv.dest);
      const exists = target && (0, _utils.isDir)(target);

      if (exists && !argv.force) {
        return (0, _utils.error)('Refusing to overwrite current directory! Please specify a different destination or use the `--force` flag', 1);
      }

      if (exists && argv.force) {
        var _ref = yield (0, _inquirer.prompt)({
          type: 'confirm',
          name: 'enableForce',
          message: `You are using '--force'. Do you wish to continue?`,
          default: false
        });

        const enableForce = _ref.enableForce;


        if (enableForce) {
          (0, _utils.info)('Initializing project in the current directory!');
        } else {
          return (0, _utils.error)('Refusing to overwrite current directory!', 1);
        }
      }

      const repo = argv.template || TEMPLATE;

      const spinner = (0, _ora2.default)({
        text: 'Fetching the template',
        color: 'magenta'
      }).start();

      // Attempt to fetch the `template`
      const archive = yield _gittar2.default.fetch(repo).catch(function (err) {
        spinner.fail('An error occured while fetching template.');
        return (0, _utils.error)((err || {}).code === 404 ? `Could not find repository: ${repo}` : (err || {}).message, 1);
      });

      spinner.text = 'Extracting the template';

      // Extract files from `archive` to `target`
      // TODO: read & respond to meta/hooks
      const keeps = [];
      yield _gittar2.default.extract(archive, target, {
        strip: 2,
        filter(path, obj) {
          if (path.includes('/template/')) {
            obj.on('end', () => {
              if (obj.type === 'File' && !isMedia(obj.path)) {
                keeps.push(obj.absolute);
              }
            });
            return true;
          }
          return false;
        }
      });

      if (keeps.length) {
        // eslint-disable-next-line
        let dict = new Map()
        // TODO: concat author-driven patterns
        ;['name'].forEach(function (str) {
          // if value is defined
          if (argv[str] !== undefined) {
            dict.set(new RegExp(`{{\\s?${str}\\s}}`, 'g'), argv[str]);
          }
        });
        // Update each file's contents
        const enc = 'utf8';
        for (var _iterator = keeps, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
          var _ref2;

          if (_isArray) {
            if (_i >= _iterator.length) break;
            _ref2 = _iterator[_i++];
          } else {
            _i = _iterator.next();
            if (_i.done) break;
            _ref2 = _i.value;
          }

          const entry = _ref2;

          let buf = yield _fs2.default.readFile(entry, enc);
          dict.forEach(function (v, k) {
            buf = buf.replace(k, v);
          });
          yield _fs2.default.writeFile(entry, buf, enc);
        }
      } else {
        return (0, _utils.error)(`No \`template\` directory found within ${repo}!`, 1);
      }

      spinner.text = 'Parsing `package.json` file';

      // Validate user's `package.json` file
      let pkgData;
      const pkgFile = (0, _path.resolve)(target, 'package.json');

      if (pkgFile) {
        pkgData = JSON.parse((yield _fs2.default.readFile(pkgFile)));
      } else {
        (0, _utils.warn)('Could not locate `package.json` file!');
      }

      if (argv.name) {
        // Update `package.json` key
        if (pkgData) {
          spinner.text = 'Updating `name` within `package.json` file';
          pkgData.name = argv.name.toLowerCase().replace(/\s+/g, '_');
          if (!pkgData.skpm) {
            pkgData.skpm = {};
          }
          pkgData.skpm.name = argv.name;
        }
        // Find a `manifest.json`; use the first match, if any
        const files = yield Promise.promisify(_glob2.default)(`${target}/**/manifest.json`);
        const manifest = files[0] && JSON.parse((yield _fs2.default.readFile(files[0])));
        if (manifest) {
          spinner.text = 'Updating `title` within `manifest.json` file';
          manifest.menu.title = argv.name;
          // Write changes to `manifest.json`
          yield _fs2.default.writeFile(files[0], JSON.stringify(manifest, null, 2));
        }
      }

      if (pkgData) {
        // Assume changes were made ¯\_(ツ)_/¯
        yield _fs2.default.writeFile(pkgFile, JSON.stringify(pkgData, null, 2));
      }

      if (argv.install) {
        spinner.text = 'Installing dependencies';
        yield (0, _setup.install)(target);
      }

      spinner.succeed('Done!\n');

      if (argv.git) {
        yield (0, _setup.initGit)(target);
      }

      return `${`
To get started, cd into the new directory:
  ${(0, _chalk.green)(`cd ${argv.dest}`)}

To start a development live-reload build:
  ${(0, _chalk.green)(`npm run start`)}

To build the plugin:
  ${(0, _chalk.green)(`npm run build`)}

To publish the plugin:
  ${(0, _chalk.green)('skpm publish')}
`}\n`;
    })();
  }
});